@@ -1,6 +1,11 @@
 # coding: utf-8
+# frozen_string_literal: true
 
 require 'mechanize/test_case'
+unless RUBY_PLATFORM == 'java'
+  require 'brotli'
+  require 'zstd-ruby'
+end
 
 class TestMechanizeHttpAgent < Mechanize::TestCase
 
@@ -29,7 +34,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
   def skip_if_jruby_zlib
     if RUBY_ENGINE == 'jruby'
-      meth = caller[0][/`(\w+)/, 1]
+      meth = caller_locations(1,1).first.base_label
       skip "#{meth}: skipped because how Zlib handles error is different in JRuby"
     end
   end
@@ -575,8 +580,8 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
   def test_request_language_charset
     @agent.request_language_charset @req
 
-    assert_equal 'en-us,en;q=0.5', @req['accept-language']
-    assert_equal 'ISO-8859-1,utf-8;q=0.7,*;q=0.7', @req['accept-charset']
+    assert_equal('en-us,en;q=0.5', @req['accept-language'])
+    assert_nil(@req['accept-charset'])
   end
 
   def test_request_referer
@@ -923,6 +928,86 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
     assert_equal 'part', body.read
   end
 
+  def test_response_content_encoding_brotli_when_brotli_not_loaded
+    skip("only test this on jruby which doesn't have brotli support") unless RUBY_ENGINE == 'jruby'
+
+    @res.instance_variable_set :@header, 'content-encoding' => %w[br]
+    body_io = StringIO.new("content doesn't matter for this test")
+
+    e = assert_raises(Mechanize::Error) do
+      @agent.response_content_encoding(@res, body_io)
+    end
+    assert_includes(e.message, "cannot deflate brotli-encoded response")
+
+    assert(body_io.closed?)
+  end
+
+  def test_response_content_encoding_brotli
+    skip("jruby does not have brotli support") if RUBY_ENGINE == 'jruby'
+
+    @res.instance_variable_set :@header, 'content-encoding' => %w[br]
+    body_io = StringIO.new(Brotli.deflate("this is compressed by brotli"))
+
+    body = @agent.response_content_encoding(@res, body_io)
+
+    assert_equal("this is compressed by brotli", body.read)
+    assert(body_io.closed?)
+  end
+
+  def test_response_content_encoding_brotli_corrupt
+    skip("jruby does not have brotli support") if RUBY_ENGINE == 'jruby'
+
+    @res.instance_variable_set :@header, 'content-encoding' => %w[br]
+    body_io = StringIO.new("not a brotli payload")
+
+    e = assert_raises(Mechanize::Error) do
+      @agent.response_content_encoding(@res, body_io)
+    end
+    assert_includes(e.message, "error inflating brotli-encoded response")
+    assert_kind_of(Brotli::Error, e.cause)
+    assert(body_io.closed?)
+  end
+
+  def test_response_content_encoding_zstd_when_zstd_not_loaded
+    skip("only test this on jruby which doesn't have zstd support") unless RUBY_ENGINE == 'jruby'
+
+    @res.instance_variable_set :@header, 'content-encoding' => %w[zstd]
+    body_io = StringIO.new("content doesn't matter for this test")
+
+    e = assert_raises(Mechanize::Error) do
+      @agent.response_content_encoding(@res, body_io)
+    end
+    assert_includes(e.message, 'cannot deflate zstd-encoded response')
+
+    assert(body_io.closed?)
+  end
+
+  def test_response_content_encoding_zstd
+    skip('jruby does not have zstd support') if RUBY_ENGINE == 'jruby'
+
+    @res.instance_variable_set :@header, 'content-encoding' => %w[zstd]
+    body_io = StringIO.new(Zstd.compress('this is compressed by zstd'))
+
+    body = @agent.response_content_encoding(@res, body_io)
+
+    assert_equal('this is compressed by zstd', body.read)
+    assert(body_io.closed?)
+  end
+
+  def test_response_content_encoding_zstd_corrupt
+    skip('jruby does not have zstd support') if RUBY_ENGINE == 'jruby'
+
+    @res.instance_variable_set :@header, 'content-encoding' => %w[zstd]
+    body_io = StringIO.new('not a zstd payload')
+
+    e = assert_raises(Mechanize::Error) do
+      @agent.response_content_encoding(@res, body_io)
+    end
+    assert_includes(e.message, 'error decompressing zstd-encoded response')
+    assert_kind_of(RuntimeError, e.cause)
+    assert(body_io.closed?)
+  end
+
   def test_response_content_encoding_gzip_corrupt
     log = StringIO.new
     logger = Logger.new log
@@ -1027,7 +1112,7 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
     body = @agent.response_content_encoding @res, body_io
 
-    expected = "test\xB2"
+    expected = +"test\xB2"
     expected.force_encoding Encoding::BINARY if have_encoding?
 
     content = body.read
@@ -1432,11 +1517,11 @@ class TestMechanizeHttpAgent < Mechanize::TestCase
 
       io = @agent.response_read res, req, uri
 
-      expected = "π\n".force_encoding(Encoding::BINARY)
+      expected = "π\n".dup.force_encoding(Encoding::BINARY)
 
       # Ruby 1.8.7 doesn't let us set the write mode of the tempfile to binary,
       # so we should expect an inserted carriage return on some platforms
-      expected_with_carriage_return = "π\r\n".force_encoding(Encoding::BINARY)
+      expected_with_carriage_return = "π\r\n".dup.force_encoding(Encoding::BINARY)
 
       body = io.read
       assert_match(/^(#{expected}|#{expected_with_carriage_return})$/m, body)
